@@ -3,7 +3,7 @@ package Test::Nginx;
 use strict;
 use warnings;
 
-our $VERSION = '0.25';
+our $VERSION = '0.26';
 
 __END__
 
@@ -35,6 +35,10 @@ L<Test::Nginx::Socket::Lua>
 
 L<Test::Nginx::Socket::Lua::Stream>
 
+=item *
+
+L<Test::Nginx::Socket::Lua::Dgram>
+
 =back
 
 =item *
@@ -89,7 +93,7 @@ L<http://wiki.nginx.org/NginxHttpMemcModule>
 
 =item ngx_drizzle
 
-L<https://github.com/chaoslawful/drizzle-nginx-module>
+L<https://github.com/openresty/drizzle-nginx-module>
 
 =item ngx_rds_json
 
@@ -109,7 +113,7 @@ L<https://github.com/openresty/srcache-nginx-module>
 
 =item ngx_lua
 
-L<https://github.com/chaoslawful/lua-nginx-module>
+L<https://github.com/openresty/lua-nginx-module>
 
 =item ngx_set_misc
 
@@ -141,7 +145,7 @@ L<https://github.com/FRiCKLE/ngx_coolkit>
 
 =item Naxsi
 
-L<http://code.google.com/p/naxsi/>
+L<https://github.com/nbs-system/naxsi>
 
 =item ngx_shibboleth
 
@@ -174,13 +178,13 @@ The C<openresty> mailing list is for Chinese speakers: L<https://groups.google.c
 
 =head1 AUTHORS
 
-Yichun Zhang (agentzh) C<< <agentzh@gmail.com> >>
+Yichun Zhang (agentzh) C<< <agentzh@gmail.com> >>, OpenResty Inc.
 
 Antoine BONAVITA C<< <antoine.bonavita@gmail.com> >>
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright (c) 2009-2016, Yichun Zhang (agentzh) C<< <agentzh@gmail.com> >>.
+Copyright (c) 2009-2017, Yichun Zhang (agentzh) C<< <agentzh@gmail.com> >>, OpenResty Inc.
 
 Copyright (c) 2011-2012, Antoine Bonavita C<< <antoine.bonavita@gmail.com> >>.
 
