@@ -3,7 +3,7 @@ package Test::Nginx::Util;
 use strict;
 use warnings;
 
-our $VERSION = '0.25';
+our $VERSION = '0.26';
 
 use base 'Exporter';
 
@@ -226,6 +226,7 @@ our @BlockPreprocessors;
 
 sub bail_out (@);
 
+our $Randomize              = $ENV{TEST_NGINX_RANDOMIZE};
 our $NginxBinary            = $ENV{TEST_NGINX_BINARY} || 'nginx';
 our $Workers                = 1;
 our $WorkerConnections      = 64;
@@ -233,13 +234,47 @@ our $LogLevel               = $ENV{TEST_NGINX_LOG_LEVEL} || 'debug';
 our $MasterProcessEnabled   = $ENV{TEST_NGINX_MASTER_PROCESS} || 'off';
 our $DaemonEnabled          = 'on';
 our $ServerPort             = $ENV{TEST_NGINX_SERVER_PORT} || $ENV{TEST_NGINX_PORT} || 1984;
-our $ServerPortForClient    = $ENV{TEST_NGINX_CLIENT_PORT} || $ENV{TEST_NGINX_PORT} || 1984;
+our $ServerPortForClient    = $ENV{TEST_NGINX_CLIENT_PORT} || $ServerPort || 1984;
 our $NoRootLocation         = 0;
 our $TestNginxSleep         = $ENV{TEST_NGINX_SLEEP} || 0.015;
 our $BuildSlaveName         = $ENV{TEST_NGINX_BUILDSLAVE};
 our $ForceRestartOnTest     = (defined $ENV{TEST_NGINX_FORCE_RESTART_ON_TEST})
                                ? $ENV{TEST_NGINX_FORCE_RESTART_ON_TEST} : 1;
 
+if ($Randomize) {
+    srand $$;
+
+    undef $ServerPort;
+
+    my $tries = 1000;
+    for (my $i = 0; $i < $tries; $i++) {
+        my $port = int(rand 60000) + 1025;
+
+        my $sock = IO::Socket::INET->new(
+            LocalAddr => $ServerAddr,
+            LocalPort => $port,
+            Proto => 'tcp',
+            Timeout => 0.1,
+        );
+
+        if (defined $sock) {
+            $sock->close();
+            $ServerPort = $port;
+            last;
+        }
+
+        if ($Verbose) {
+            warn "Try again, port $port is already in use: $@\n";
+        }
+    }
+
+    if (!defined $ServerPort) {
+        bail_out "Cannot find an available listening port number after $tries attempts.\n";
+    }
+
+    $ServerPortForClient = $ServerPort;
+}
+
 our $ChildPid;
 our $UdpServerPid;
 our $TcpServerPid;
@@ -435,7 +470,16 @@ sub add_block_preprocessor(&) {
 #our ($PrevRequest)
 our $PrevConfig;
 
-our $ServRoot   = $ENV{TEST_NGINX_SERVROOT} || File::Spec->catfile(cwd() || '.', 't/servroot');
+our $ServRoot;
+
+if ($Randomize) {
+    $ServRoot = File::Spec->rel2abs("t/servroot_" . $ServerPort);
+
+} else {
+    $ServRoot = $ENV{TEST_NGINX_SERVROOT} || File::Spec->rel2abs('t/servroot');
+}
+$ENV{TEST_NGINX_SERVER_ROOT} = $ServRoot;
+
 our $LogDir     = File::Spec->catfile($ServRoot, 'logs');
 our $ErrLogFile = File::Spec->catfile($LogDir, 'error.log');
 our $AccLogFile = File::Spec->catfile($LogDir, 'access.log');
@@ -477,8 +521,8 @@ sub bail_out (@) {
     Test::More::BAIL_OUT(@_);
 }
 
-sub kill_process ($$) {
-    my ($pid, $wait) = @_;
+sub kill_process ($$$) {
+    my ($pid, $wait, $name) = @_;
 
     if ($wait) {
         eval {
@@ -500,16 +544,15 @@ sub kill_process ($$) {
             alarm 0;
         };
 
-        if (!$@) {
-            return;
-        }
-
-        if ($Verbose) {
-            warn "WARNING: child process $pid timed out.\n";
+        if ($@) {
+            if ($Verbose) {
+                warn "$name - WARNING: child process $pid timed out.\n";
+            }
         }
     }
 
     my $i = 1;
+    my $step = $TestNginxSleep;
     while ($i <= 20) {
         #warn "ps returns: ", system("ps -p $pid > /dev/stderr"), "\n";
         #warn "$pid is running? ", is_running($pid) ? "Y" : "N", "\n";
@@ -526,38 +569,49 @@ sub kill_process ($$) {
             warn "WARNING: failed to send term signal to the child process with PID $pid.\n";
         }
 
-        sleep $TestNginxSleep * $i;
+        $step *= 1.2;
+        $step = 0.5 if $step > 0.5;
+        sleep $step;
 
     } continue {
         $i++;
     }
 
     #system("ps aux|grep $pid > /dev/stderr");
-    warn "WARNING: killing the child process $pid with force...";
+    warn "$name - WARNING: killing the child process $pid with force...";
 
     kill(SIGKILL, $pid);
     waitpid($pid, 0);
 
-    sleep $TestNginxSleep;
+    if (is_running($pid)) {
+        local $SIG{ALRM} = sub { die "alarm\n" };
+        alarm timeout();
+        waitpid($pid, 0);
+        alarm 0;
+    }
 }
 
 sub cleanup () {
+    if ($Verbose) {
+        warn "cleaning up everything";
+    }
+
     for my $hdl (@CleanupHandlers) {
        $hdl->();
     }
 
     if (defined $UdpServerPid) {
-        kill_process($UdpServerPid, 1);
+        kill_process($UdpServerPid, 1, "cleanup");
         undef $UdpServerPid;
     }
 
     if (defined $TcpServerPid) {
-        kill_process($TcpServerPid, 1);
+        kill_process($TcpServerPid, 1, "cleanup");
         undef $TcpServerPid;
     }
 
     if (defined $ChildPid) {
-        kill_process($ChildPid, 1);
+        kill_process($ChildPid, 1, "cleanup");
         undef $ChildPid;
     }
 }
@@ -950,6 +1004,10 @@ sub get_nginx_version () {
         $NginxRawVersion = "$1.$2.$3";
         return get_canon_version($1, $2, $3);
     }
+    if ($out =~ m{\w+/(\d+)\.(\d+)\.(\d+)}s) {
+        $NginxRawVersion = "$1.$2.$3";
+        return get_canon_version($1, $2, $3);
+    }
     bail_out("Failed to parse the output of \"nginx -V\": $out\n");
 }
 
@@ -1145,6 +1203,9 @@ sub run_udp_server_tests ($$$) {
 
 sub run_test ($) {
     my $block = shift;
+
+    return if defined $block->SKIP;
+
     my $name = $block->name;
 
     my $first_time;
@@ -1423,25 +1484,7 @@ sub run_test ($) {
 
                 if ($UseHup) {
                     if ($first_time) {
-                        if ($Verbose) {
-                            warn "sending QUIT signal to $pid\n";
-                        }
-
-                        if (kill(SIGQUIT, $pid) == 0) { # send quit signal
-                            #warn("$name - Failed to send quit signal to the nginx process with PID $pid");
-                        }
-
-                        my $max_i = 15;
-                        for (my $i = 1; $i <= $max_i; $i++) {
-                            last unless is_running($pid);
-
-                            sleep $TestNginxSleep;
-                            next if $i < $max_i;
-
-                            warn "WARNING: $name - killing nginx $pid with force...";
-                            kill(SIGKILL, $pid);
-                            waitpid($pid, 0);
-                        }
+                        kill_process($pid, 1, $name);
 
                         undef $nginx_is_running;
                         goto start_nginx;
@@ -1489,25 +1532,7 @@ sub run_test ($) {
                     }
                 }
 
-                if ($Verbose) {
-                    warn "sending QUIT signal to $pid\n";
-                }
-
-                if (kill(SIGQUIT, $pid) == 0) { # send quit signal
-                    #warn("$name - Failed to send quit signal to the nginx process with PID $pid");
-                }
-
-                my $max_i = 15;
-                for (my $i = 1; $i <= $max_i; $i++) {
-                    last unless is_running($pid);
-
-                    sleep $TestNginxSleep;
-                    next if $i < $max_i;
-
-                    warn "WARNING: $name - killing nginx $pid with force...";
-                    kill(SIGKILL, $pid);
-                    waitpid($pid, 0);
-                }
+                kill_process($pid, 1, $name);
 
                 undef $nginx_is_running;
 
@@ -1738,15 +1763,15 @@ RUN_AGAIN:
                     if ($status == 0) {
                         warn("WARNING: $name - nginx must die but it does ",
                              "not; killing it (req $i)");
-                        my $max_i = 15;
-                        for (my $i = 1; $i <= $max_i; $i++) {
+                        my $tries = 15;
+                        for (my $i = 1; $i <= $tries; $i++) {
                             if (-f $PidFile) {
                                 last;
                             }
                             sleep $TestNginxSleep;
                         }
                         my $pid = get_pid_from_pidfile($name);
-                        kill_process($pid, 1);
+                        kill_process($pid, 1, $name);
                     }
 
                     goto RUN_AGAIN if ++$i < $RepeatEach;
@@ -1871,7 +1896,7 @@ request:
                                     }
 
                                     warn "WARNING: killing process $pid listening on target $target.\n";
-                                    kill_process($pid, 1);
+                                    kill_process($pid, 1, $name);
                                 }
                             }
                         }
@@ -1938,13 +1963,13 @@ request:
                     }
                 }
 
+                my $buf;
+
                 unless ($no_read) {
                     if ($Verbose) {
                         warn "TCP server reading request...\n";
                     }
 
-                    my $buf;
-
                     while (1) {
                         my $b;
                         my $ret = $client->recv($b, 4096);
@@ -1996,7 +2021,17 @@ request:
                             warn "TCP server writing reply...\n";
                         }
 
+                        my $ref = ref $reply;
+                        if ($ref && $ref eq 'CODE') {
+                            $reply = $reply->($buf);
+                            $ref = ref $reply;
+                        }
+
                         if (ref $reply) {
+                            if ($ref ne 'ARRAY') {
+                                bail_out('bad --- tcp_reply value');
+                            }
+
                             for my $r (@$reply) {
                                 if ($Verbose) {
                                     warn "sending reply $r";
@@ -2229,7 +2264,7 @@ request:
 
         if (defined $udp_socket) {
             if (defined $UdpServerPid) {
-                kill_process($UdpServerPid, 1);
+                kill_process($UdpServerPid, 1, $name);
                 undef $UdpServerPid;
             }
 
@@ -2247,7 +2282,7 @@ request:
                 if ($Verbose) {
                     warn "killing TCP server, pid $TcpServerPid\n";
                 }
-                kill_process($TcpServerPid, 1);
+                kill_process($TcpServerPid, 1, $name);
                 undef $TcpServerPid;
             }
 
@@ -2339,54 +2374,48 @@ END {
 
     if ($UseStap || $UseValgrind || !$ENV{TEST_NGINX_NO_CLEAN}) {
         local $?; # to avoid confusing Test::Builder::_ending
-        if (-f $PidFile) {
+        if (defined $PidFile && -f $PidFile) {
             my $pid = get_pid_from_pidfile('');
             if (!$pid) {
                 bail_out "No pid found.";
             }
             if (is_running($pid)) {
-                if ($Verbose) {
-                    warn "sending QUIT signal to $pid";
-                }
-
-                if (kill(SIGQUIT, $pid) == 0) { # send quit signal
-                    #warn("Failed to send quit signal to the nginx process with PID $pid");
-                }
-
-                my $max_i = 15;
-                for (my $i = 1; $i <= $max_i; $i++) {
-                    last unless is_running($pid);
-
-                    sleep $TestNginxSleep;
-                    next if $i < $max_i;
-
-                    warn "WARNING: killing nginx $pid with force...";
-                    kill(SIGKILL, $pid);
-                    waitpid($pid, 0);
-                }
+                kill_process($pid, 1, "END");
 
             } else {
                 unlink $PidFile;
             }
         }
     }
+
+    if ($Randomize) {
+        if (defined $ServRoot && -d $ServRoot && $ServRoot =~ m{/t/servroot_\d+}) {
+            system("rm -rf $ServRoot");
+        }
+    }
 }
 
 # check if we can run some command
 sub can_run {
     my ($cmd) = @_;
 
-    #warn "can run: @_\n";
-    #my $_cmd = $cmd;
-    #return $_cmd if (-x $_cmd or $_cmd = MM->maybe_command($_cmd));
+    #warn "can run: $cmd\n";
+    if ($cmd =~ m{[/\\]}) {
+       if (-f $cmd && -x $cmd) {
+           return $cmd;
+        }
+
+        return undef;
+    }
 
     for my $dir ((split /$Config::Config{path_sep}/, $ENV{PATH}), '.') {
         next if $dir eq '';
-        my $abs = File::Spec->catfile($dir, $_[0]);
+        my $abs = File::Spec->catfile($dir, $cmd);
+        #warn $abs;
         return $abs if -f $abs && -x $abs;
     }
 
-    return;
+    return undef;
 }
 
 1;
